﻿/* RandomNumberPusher : Generates random number with random Thread.Sleep.
 *                      Part of "SLMultiThread" demo
 *                      (c) apr 2010 Olivier Dahan - wwww.e-naxos.com/blog
 */

using System;
using System.ComponentModel;
using System.Threading;
using SLMultiThread.ThreadTools;

namespace SLMultiThread.Models
{
    /// <summary>
    /// A random number generator working in the background
    /// </summary>
    public class RandomNumberPusher : PropertyChangedBase
    {
        readonly Random rand = new Random(DateTime.Now.Millisecond);
        private readonly BackgroundWorker worker;
        private readonly object locker = new object();
        private int number;
        private bool isRunning;

        /// <summary>
        /// Initializes a new instance of the <see cref="RandomNumberPusher"/> class
        /// and launches the background worker.
        /// </summary>
        public RandomNumberPusher()
        {
            worker = new BackgroundWorker
            {
                WorkerSupportsCancellation = true
            };
            worker.DoWork += worker_DoWork;
            worker.RunWorkerAsync();
        }

        /// <summary>
        /// Gets the current random number.
        /// </summary>
        /// <value>The number.</value>
        public int Number
        {
            get
            {
                lock (locker)
                {
                    return number;
                }
            }
        }

        /// <summary>
        /// Gets a value indicating whether this instance is running.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is running; otherwise, <c>false</c>.
        /// </value>
        public bool IsRunning
        {
            get { lock (locker) { return isRunning; } }
        }

        /// <summary>
        /// Stops this instance.
        /// A stopped instance can be reused and must be sent to GC.
        /// </summary>
        public void Stop()
        {
            worker.CancelAsync();
        }


        // Background job
        private void worker_DoWork(object sender, DoWorkEventArgs e)
        {
            while (true)
            {
                if ((sender as BackgroundWorker).CancellationPending)
                {
                    lock (locker)
                    {
                        isRunning = false;
                        NotifyPropertyChanged("IsRunning");
                    }
                    return;
                }
                lock (locker)
                {
                    number = rand.Next();
                }
                NotifyPropertyChanged("Number");
                Thread.Sleep(rand.Next(200, 600));
            }
        }
       
    }
}
